import { DeleteResult, EntityManager, InsertResult } from "typeorm";
import { dataSource } from "../../../data-source";
import { rolePermission } from "../../../entity/user/role_permission";
import InternalException from "../../../exceptions/internalException";
import RoleService from "../../../service/user/roleService";
import {
  CreateRolePermissionCommand,
  DeleteRolePermissionCommand,
  UpdateRolePermissionsCommand,
} from "./rolePermissionCommand";
import PermissionHelper from "../../../helpers/permissionHelper";
import RolePermissionService from "../../../service/user/rolePermissionService";
import { PermissionString } from "../../../type/permissionTypes";

export default abstract class RolePermissionCommandHandler {
  /**
   * @description update role permissions
   * @param {UpdateRolePermissionsCommand} updateRolePermissions
   * @returns {Promise<void>}
   */
  static async updatePermissions(updateRolePermissions: UpdateRolePermissionsCommand): Promise<void> {
    let currentPermissions = (await RolePermissionService.getByRole(updateRolePermissions.roleId)).map(
      (r) => r.permission
    );
    return await dataSource.manager
      .transaction(async (manager) => {
        let newPermissions = PermissionHelper.getWhatToAdd(currentPermissions, updateRolePermissions.permissions);
        let removePermissions = PermissionHelper.getWhatToRemove(currentPermissions, updateRolePermissions.permissions);
        if (newPermissions.length != 0) {
          await this.updatePermissionsAdd(manager, updateRolePermissions.roleId, newPermissions);
        }
        if (removePermissions.length != 0) {
          await this.updatePermissionsRemove(manager, updateRolePermissions.roleId, removePermissions);
        }
      })
      .then(() => {})
      .catch((err) => {
        throw new InternalException("Failed saving role permissions", err);
      });
  }

  private static async updatePermissionsAdd(
    manager: EntityManager,
    roleId: number,
    permissions: Array<PermissionString>
  ): Promise<InsertResult> {
    return await manager
      .createQueryBuilder()
      .insert()
      .into(rolePermission)
      .values(
        permissions.map((p) => ({
          permission: p,
          roleId: roleId,
        }))
      )
      .orIgnore()
      .execute();
  }

  private static async updatePermissionsRemove(
    manager: EntityManager,
    roleId: number,
    permissions: Array<PermissionString>
  ): Promise<DeleteResult> {
    return await manager
      .createQueryBuilder()
      .delete()
      .from(rolePermission)
      .where("roleId = :id", { id: roleId })
      .andWhere("permission IN (:...permission)", { permission: permissions })
      .execute();
  }

  /**
   * @description grant permission to user
   * @param {CreateRolePermissionCommand} createPermission
   * @returns {Promise<number>}
   */
  static async create(createPermission: CreateRolePermissionCommand): Promise<number> {
    return await dataSource
      .createQueryBuilder()
      .insert()
      .into(rolePermission)
      .values({
        permission: createPermission.permission,
        role: await RoleService.getById(createPermission.roleId),
      })
      .execute()
      .then((result) => {
        return result.identifiers[0].id;
      })
      .catch((err) => {
        throw new InternalException("Failed saving role permission", err);
      });
  }

  /**
   * @description remove permission from role
   * @param {DeleteRolePermissionCommand} deletePermission
   * @returns {Promise<any>}
   */
  static async delete(deletePermission: DeleteRolePermissionCommand): Promise<any> {
    return await dataSource
      .createQueryBuilder()
      .delete()
      .from(rolePermission)
      .where("roleId = :id", { id: deletePermission.roleId })
      .andWhere("permission = :permission", { permission: deletePermission.permission })
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new InternalException("failed role permission removal", err);
      });
  }
}
